/*
*   Copyright (c) 2000 Martin Newstead (mseries@brundell.fsnet.co.uk).  All Rights Reserved.
*
*   The author makes no representations or warranties about the suitability of the
*   software, either express or implied, including but not limited to the
*   implied warranties of merchantability, fitness for a particular
*   purpose, or non-infringement. The author shall not be liable for any damages
*   suffered by licensee as a result of using, modifying or distributing
*   this software or its derivatives.
*
*   The author requests that he be notified of any application, applet, or other binary that
*   makes use of this code and that some acknowedgement is given. Comments, questions and
*   requests for change will be welcomed.
*/
package mseries.ui;

import mseries.Calendar.MDateChanger;
import mseries.Calendar.MDefaultPullDownConstraints;
import mseries.Calendar.MFieldListener;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.ResourceBundle;

public class MDateEntryFieldDemo implements Runnable
{

    JFrame frame;

    public static void main(String[] argv)
    {
        MDateEntryFieldDemo demo = new MDateEntryFieldDemo();
        demo.run();
    }

    public void run()
    {
        String build = "";
        String date = "";

//        Locale.setDefault(new Locale("da", "DA"));
        /*
        *   This block of code deals with placing the build number & date in the
        *   window title.
        */
        try
        {
            ResourceBundle rb = ResourceBundle.getBundle("build");
            build = rb.getString("build.number");
            date = rb.getString("build.date");

        }
        catch (Exception e)
        {
            System.out.println(e);
        }
        frame = new JFrame("Test Window " + build + " " + date);

        frame.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                System.exit(0);
            }
        });

        setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

        frame.setJMenuBar(createMenus());




        /*****************************
         * The Demo code starts here  *
         *****************************/

        /*---------------- MDateEntryField ------------------*/
        Calendar cx = Calendar.getInstance();
        cx.set(2001, 5, 12);
        DateFormat d=DateFormat.getDateInstance(DateFormat.SHORT);
        d.setLenient(false);
        final MDateEntryField entryField = new MDateEntryField(d);

        entryField.setNullOnEmpty(true);
        entryField.setFocusLostBehavior(JFormattedTextField.PERSIST);
        entryField.setValue(cx.getTime());

        MDefaultPullDownConstraints c = new MDefaultPullDownConstraints();
        c.firstDay = Calendar.MONDAY;
        //c.changerStyle = MDateChanger.BUTTON;
        c.changerStyle=MDateChanger.SPINNER;
//        c.changerStyle=MDateChanger.SCROLLBAR;
        c.hasShadow = true;
        c.selectionEventsEnabled=true;
        //c.todayForeground=Color.red;
        //c.todayBackground=Color.green;
        //c.imageFile="d:\\temp\\fineview.jpg";

        // Set the background colours of the pull down
        //c.background=Color.red;
        // Set the background of the entry field
        //entryField.setBackground(Color.red);
        //entryField.setShowTodayButton(true, true);

        entryField.setConstraints(c);
//        entryField.setEditable(false);
//        entryField.setEnabled(false);

        entryField.addMFieldListener(new MFieldListener()
        {
            public void fieldEntered(FocusEvent e)
            {
                System.out.println("MDateEntryField:Entered");
            }

            public void fieldExited(FocusEvent e)
            {
                System.out.println("MDateEntryField:Exited");
                try
                {
                    System.out.println("EE " + entryField.getValue());
                }
                catch (Exception ee)
                {
                    System.out.println(ee);
                }

            }
        });

        entryField.addMChangeListener(new MChangeListener()
        {
            public void valueChanged(MChangeEvent e)
            {
                System.out.println(e.getValue());
            }
        });

        entryField.setToolTipText("Click the button");

        /*---------------- MDateField ------------------*/
        DateFormat d2=DateFormat.getDateInstance(DateFormat.SHORT);
        d.setLenient(true);
        MDateField endDate = new MDateField(d2);
        MDefaultPullDownConstraints c2 = new MDefaultPullDownConstraints();
        c2.firstDay = Calendar.SUNDAY;
        //c2.background=Color.red;
        //c2.outOfRangeBackground=Color.green;
        //c2.outOfRangeForeground=Color.yellow;
        endDate.setConstraints(c2);
        endDate.setPopup(true);
        endDate.setToolTipText("Right click me");

        /*---------------- Test combobox ----------------*/
        String[] items = {"England", "Scotland", "N. Ireland", "Wales"};
        JComboBox c1 = new JComboBox(items);



        /*---------------- Layout the frame ----------------*/
        GridLayout l = new GridLayout(3, 0, 7, 7);
        JPanel displayPanel = new JPanel(l);

        //displayPanel.add(new JTextField("Hello"));

        displayPanel.add(new JLabel("MDateEntryField"));
        displayPanel.add(entryField);
        displayPanel.add(new JLabel("MDateSelector (Right Click)"));
        displayPanel.add(endDate);

        displayPanel.add(new JLabel("Test Combo"));
        displayPanel.add(c1);

        frame.getContentPane().setLayout(new FlowLayout());
        frame.getContentPane().add(displayPanel);

        //JButton set=new JButton("Set");
        //set.addActionListener(new ActionListener(){
        //    public void actionPerformed(ActionEvent e)
        //    {
        //        Date end=new Date(105, 11, 31);
        //        Date start=new Date (90, 0, 1);
        //        entryField.setMinimum(start);
        //        entryField.setMaximum(end);
        //    }
        //});

//frame.getContentPane().add(set);

        frame.pack();
        frame.show();
    }


    private ButtonGroup lafMenuGroup = new ButtonGroup();

    public JMenuBar createMenus()
    {
        JMenuBar menuBar = new JMenuBar();
        JMenu lafMenu;
        /* LAF Switching Menu */
        UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
        if (lafs.length <= 0)
        {
            return menuBar;
        }
        lafMenu = menuBar.add(new JMenu("Look & Feel"));
        lafMenu.setMnemonic('L');

        for (int i = 0; i < lafs.length; i++)
        {
            UIManager.LookAndFeelInfo laf = lafs[i];

            String lafName = laf.getName();
            String lafClassName = laf.getClassName();
            createLafMenuItem(lafMenu, lafName, "", lafClassName);
        }
        createLafMenuItem(lafMenu, "Kunststoff", "", "com.incors.plaf.kunststoff.KunststoffLookAndFeel");
        createLafMenuItem(lafMenu, "JGoodies PlasticXP", "", "com.jgoodies.plaf.plastic.PlasticXPLookAndFeel");
        return menuBar;
    }

    public JMenuItem createLafMenuItem(JMenu menu, String label, String mnemonic, String laf)
    {
        JMenuItem mi = menu.add(new JRadioButtonMenuItem(label));
        lafMenuGroup.add(mi);
        mi.addActionListener(new ChangeLookAndFeelAction(this, laf));

        return mi;
    }

    public void setLookAndFeel(String laf)
    {
        try
        {
            UIManager.setLookAndFeel(laf);
            SwingUtilities.updateComponentTreeUI(frame);
            frame.pack();
        }
        catch (Exception ex)
        {
            System.out.println("Failed loading L&F: " + laf);
            JOptionPane.showMessageDialog(null,  laf +" is not on the classpath", "Un-supported Look & Feel", JOptionPane.ERROR_MESSAGE);
        }
    }
    public class MSeriesTestInputVerifier extends InputVerifier
    {
        public boolean verify(JComponent input)
        {
            boolean retval = false;
            System.out.println("input.getClass() = " + input.getClass());
            // the getName call returns null
            System.out.println("input.getName() = " + input.getName());
            // the getClientProperty call returns null
            System.out.println("input.getClientProperty(MNEMONIC) = " + input.getClientProperty("MNEMONIC"));
            if (input instanceof MDateEntryField)
            {
                System.out.println("MDateEntryField");
                retval = true;
            }
            else if (input instanceof JTextField)
            {
                System.out.println("JTextField");
                retval = true;
            }
            return retval;
        }

    }
    class ChangeLookAndFeelAction implements ActionListener
    {
        MDateEntryFieldDemo m;
        String laf;

        public ChangeLookAndFeelAction(MDateEntryFieldDemo d, String laf)
        {
            m = d;
            this.laf = laf;
        }

        public void actionPerformed(ActionEvent e)
        {
            m.setLookAndFeel(laf);
        }
    }
}


