/*
*   Copyright (c) 2000 Martin Newstead (mseries@brundell.fsnet.co.uk).  All Rights Reserved.
*
*   The author makes no representations or warranties about the suitability of the
*   software, either express or implied, including but not limited to the
*   implied warranties of merchantability, fitness for a particular
*   purpose, or non-infringement. The author shall not be liable for any damages
*   suffered by licensee as a result of using, modifying or distributing
*   this software or its derivatives.
*
*   The author requests that he be notified of any application, applet, or other binary that
*   makes use of this code and that some acknowedgement is given. Comments, questions and
*   requests for change will be welcomed.
*/
package mseries.ui;

import mseries.Calendar.MDateChanger;
import mseries.Calendar.MDefaultPullDownConstraints;
import mseries.Calendar.MFieldListener;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;

public class MDateEntryFieldDemo2 implements Runnable
{

    JFrame frame;

    public static void main(String[] argv)
    {
        MDateEntryFieldDemo2 demo = new MDateEntryFieldDemo2();
        demo.run();
    }

    public void run()
    {

        //Locale.setDefault(new Locale("ru", "RU"));
        /*
        *   This block of code deals with placing the build number & date in the
        *   window title.
        */

        frame = new JFrame("Test Window ");

        frame.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                System.exit(0);
            }
        });

        setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

        //frame.setJMenuBar(createMenus());




        /*****************************
         * The Demo code starts here  *
         *****************************/

        /*---------------- MDateEntryField ------------------*/
        Calendar cx = Calendar.getInstance();
        cx.set(2001, 5, 12);
        final MDateEntryField entryField = new MDateEntryField(10);

        //entryField.setDateFormatter(new MSimpleDateFormat("dd/MM/yyyy"));
        //entryField.setValue(cx.getTime());

        MDefaultPullDownConstraints c = new MDefaultPullDownConstraints();
        c.firstDay = Calendar.MONDAY;
        c.changerStyle = MDateChanger.BUTTON;
        //c.changerStyle=MDateChanger.SPINNER;
        //c.changerStyle=MDateChanger.SCROLLBAR;
        c.hasShadow = true;
        //c.selectionEventsEnabled=false;
        //c.todayForeground=Color.red;
        //c.todayBackground=Color.green;
        //c.imageFile="d:\\temp\\fineview.jpg";

        // Set the background colours of the pull down
        //c.background=Color.red;
        // Set the background of the entry field
        //entryField.setBackground(Color.red);
        //entryField.setShowTodayButton(true, true);

        entryField.setConstraints(c);
//        entryField.setEditable(false);
//        entryField.setEnabled(false);

        entryField.addMFieldListener(new MFieldListener()
        {
            public void fieldEntered(FocusEvent e)
            {
                System.out.println("MDateEntryField:Entered");
            }

            public void fieldExited(FocusEvent e)
            {
                System.out.println("MDateEntryField:Exited");
                try
                {
                    System.out.println("EE " + entryField.getValue());
                }
                catch (Exception ee)
                {
                    System.out.println(ee);
                }

            }
        });

        entryField.addMChangeListener(new MChangeListener()
        {
            public void valueChanged(MChangeEvent e)
            {
                System.out.println(e.getValue());
            }
        });

        entryField.setToolTipText("Click the button");


        /*---------------- Layout the frame ----------------*/
        GridLayout l = new GridLayout(3, 4, 7, 7);
        JPanel displayPanel = new JPanel(l);


        displayPanel.add(new JLabel("MDateEntryField"));
        displayPanel.add(entryField);

        frame.getContentPane().setLayout(new FlowLayout());
        frame.getContentPane().add(displayPanel);


        frame.pack();
        frame.show();
    }


    private ButtonGroup lafMenuGroup = new ButtonGroup();

    public JMenuBar createMenus()
    {
        JMenuBar menuBar = new JMenuBar();
        JMenu lafMenu;
        /* LAF Switching Menu */
        UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
        if (lafs.length <= 0)
        {
            return menuBar;
        }
        lafMenu = menuBar.add(new JMenu("Look & Feel"));
        lafMenu.setMnemonic('L');

        for (int i = 0; i < lafs.length; i++)
        {
            UIManager.LookAndFeelInfo laf = lafs[i];

            String lafName = laf.getName();
            String lafClassName = laf.getClassName();
            createLafMenuItem(lafMenu, lafName, "", lafClassName);
        }
        return menuBar;
    }

    public JMenuItem createLafMenuItem(JMenu menu, String label, String mnemonic, String laf)
    {
        JMenuItem mi = menu.add(new JRadioButtonMenuItem(label));
        lafMenuGroup.add(mi);
        mi.addActionListener(new ChangeLookAndFeelAction(this, laf));

        return mi;
    }

    public void setLookAndFeel(String laf)
    {
        try
        {
            UIManager.setLookAndFeel(laf);
            SwingUtilities.updateComponentTreeUI(frame);
            frame.pack();
        }
        catch (Exception ex)
        {
            System.out.println("Failed loading L&F: " + laf);
            System.out.println(ex);
        }
    }

    class ChangeLookAndFeelAction implements ActionListener
    {
        MDateEntryFieldDemo2 m;
        String laf;

        public ChangeLookAndFeelAction(MDateEntryFieldDemo2 d, String laf)
        {
            m = d;
            this.laf = laf;
        }

        public void actionPerformed(ActionEvent e)
        {
            m.setLookAndFeel(laf);
        }
    }
}


